;;; **********************************************************************
(in-package :cm)

; (load (compile-file "hd:desktop folder:tests.lisp"))
; (test-runtime ':seconds)
; (test-runtime ':milliseconds)

(defparameter p nil)
(defparameter f nil)

(defun ff-midi-write-message (m d) m d 0) ; stub out

(defun sel (a b)
  (loop for n = (score-select a b)
        while n do nil))

(defun itm (s n)
  (loop repeat n do (item s)))

(defun env (x e n)
  (loop repeat n do (interp x e)))

(eval-when (load compile eval)
  (unless (fboundp 'clock-mode ) 
    (defun clock-mode (&rest args) args :seconds)
    (defvar *random-range* nil)
    (setf *post-processing* nil)
    (defvar *old-cm* t)))

(defun test-cm (&optional (test t) &key (repeat 4) (dribble nil) 
                     &aux thread algo merge offset
                     stream (envl '(0 1 1 0)))
  (labels ((test? (n n2) (eql (first n) n2))
           (setmode (m)
             (clock-mode m)
	     (ecase m
	       (:seconds
	        (setf thread #!ft algo #!fa merge #!fm offset 0.0))
	       (:milliseconds 
	        (setf thread #!it algo #!ia merge #!im offset 0)))))
    (when dribble (dribble dribble))
    (if (eq test t)
      (setf test
            (if *old-cm*
              (loop for i from 1 to 16 collect i)
              '(1 3 5 7 9 11 12 13 15 16)))
      (unless (consp test) (setf test (list test))))
    (when (test? test 1)
      (setmode :seconds)
      (format t "~&~&Test ~D: ~Ax ~A algo selection." 
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (sel thread offset)) (terpri)))

    (when (test? test 2)
      (setmode :milliseconds)
      (format t "~&~&Test ~D: ~Ax ~A algo selection."
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (sel thread offset)) (terpri)))

    (when (test? test 3)
      (setmode :seconds)
      (format t "~&~&Test ~D: ~Ax ~A algo selection." 
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (sel algo offset)) (terpri)))
 
    (when (test? test 4)
      (setmode :milliseconds)
      (format t "~&~&Test ~D: ~Ax ~A algo selection." 
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (sel algo offset)) (terpri)))
 
    (when (test? test 5)
      (setmode :seconds)
      (format t "~&~&Test ~D: ~Ax ~A merge selection." 
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (sel merge offset)) (terpri)))

    (when (test? test 6)
      (setmode :milliseconds)
      (format t "~&~&Test ~D: ~Ax ~A merge selection." 
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (sel merge offset)) (terpri)))

    (when (test? test 7)
      (setmode :seconds)
      (format t "~&~&Test ~D: ~Ax ~A midi-port algorithm write-event."
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (mix-object algo p offset)) (terpri)))

    (when (test? test 8)
      (setmode :milliseconds)
      (format t "~&~&Test ~D: ~Ax ~A midi-port algorithm write-event."
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (mix-object algo p offset)) (terpri)))

    (when (test? test 9)
      (setmode :seconds)
      (format t "~&~&Test ~D: ~Ax ~A midi-file algorithm write-event."
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (mix-object algo f offset)) (terpri)))

    (when (test? test 10)
      (setmode :seconds)
      (format t "~&~&Test ~D: ~Ax ~A midi-file algorithm write-event."
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (mix-object algo f offset)) (terpri)))

    (when (test? test 11)
      (setf stream (items a b c d e))
      (format t "~&~&Test ~D: ~Ax cyclic items." (pop test) repeat)
      (dotimes (i repeat) (time (itm stream 1000))))

    (when (test? test 12)
      (setf stream (notes a b c d e in heap))
      (format t "~&~&Test ~D: ~Ax heap notes." (pop test) repeat)
      (dotimes (i repeat) (time (itm stream 1000))))

    (when (test? test 13)
      (setmode :seconds)
      (setf stream (rhythms q e e. t16 56 in random tempo 123))
      (format t "~&~&Test ~D: ~Ax ~A random rhythms tempo 123 "
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (itm stream 1000))))
     
    (when (test? test 14)
      (setmode :milliseconds)
      (setf *random-range* most-positive-fixnum)
      (setf stream (rhythms q e e. t16 56 in random tempo 123))
      (format t "~&~&Test ~D: ~Ax ~A random rhythms tempo 123 " 
              (pop test) repeat (clock-mode))
      (dotimes (i repeat) (time (itm stream 1000)))
      (setf *random-range* nil))

    (when (test? test 15)
      (format t "~&~&Test ~D: ~Ax floating point interp " 
              (pop test) repeat)
      (dotimes (i repeat) (time (env .666 envl 1000))))

    (when (test? test 16)
      (format t "~&~&Test ~D: ~Ax rational interp "
              (pop test) repeat)
      (dotimes (i repeat) (time (env 2/3 envl 1000))))
    (setmode :seconds)
    (when dribble (dribble))
    ))

(eval-when (load eval)
  (setf p (open-event-stream 'midi-port :open 'port 'a))
  (setf f (open-event-stream 'midi-file :default 'name "testw.midi"
                             'play nil))

  (thread ft ()
    (dotimes (i 1000)
      (object midi-note rhythm .1 duration .1 amplitude .2 note 60)))

  (algorithm fa midi-note (length 1000  rhythm .1 duration .1 amplitude
                                  .5 note 60))
  (merge fm () #!fa)
  
  (thread it ()
    (dotimes (i 1000)
      (object midi-note rhythm 100 duration 100 amplitude 64 note 60)))
  
  (algorithm ia midi-note (length 1000  rhythm 100 duration 100 
                                  amplitude 64 note 60))
  (merge im () #!ia )

  ;; hey tobias why do i need to do this stub at load time???
  #+mcl (defun %midi-write-message (m d) m d 0)
  
  )

